@@ -2,7 +2,7 @@
  * rofi
  *
  * MIT/X11 License
- * Copyright © 2013-2021 Qball Cow <qball@gmpclient.org>
+ * Copyright © 2013-2023 Qball Cow <qball@gmpclient.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -25,7 +25,7 @@
  *
  */
 
-#include <config.h>
+#include "config.h"
 #include <glib.h>
 #include <widgets/box.h>
 #include <widgets/icon.h>
@@ -36,6 +36,7 @@
 
 #include "settings.h"
 #include "theme.h"
+#include "view.h"
 
 #include "timings.h"
 
@@ -71,6 +72,9 @@ struct _listview {
   // RChanged
   // Text needs to be repainted.
   unsigned int rchanged;
+
+  // The direction we pack the widgets.
+  RofiOrientation pack_direction;
   // Administration
 
   unsigned int cur_page;
@@ -95,8 +99,10 @@ struct _listview {
   unsigned int dynamic;
   unsigned int eh;
   unsigned int reverse;
+  gboolean require_input;
+  gboolean filtered;
+
   gboolean cycle;
-  gboolean multi_select;
 
   ScrollType scroll_type;
 
@@ -106,11 +112,16 @@ struct _listview {
   listview_update_callback callback;
   void *udata;
 
+  listview_selection_changed_callback sc_callback;
+  void *sc_udata;
+
   gboolean scrollbar_scroll;
 
   xcb_timestamp_t last_click;
   listview_mouse_activated_cb mouse_activated;
   void *mouse_activated_data;
+  
+  listview_page_changed_cb page_callback;
 
   char *listview_name;
 
@@ -159,24 +170,49 @@ static void listview_set_state(_listview_row r, TextBoxFontType tbft) {
 }
 static void listview_add_widget(listview *lv, _listview_row *row, widget *wid,
                                 const char *label) {
-  TextboxFlags flags = (lv->multi_select) ? TB_INDICATOR : 0;
   if (strcasecmp(label, "element-icon") == 0) {
     row->icon = icon_create(WIDGET(wid), "element-icon");
     box_add((box *)wid, WIDGET(row->icon), FALSE);
   } else if (strcasecmp(label, "element-text") == 0) {
     row->textbox =
         textbox_create(WIDGET(wid), WIDGET_TYPE_TEXTBOX_TEXT, "element-text",
-                       TB_AUTOHEIGHT | flags, NORMAL, "DDD", 0, 0);
+                       TB_AUTOHEIGHT, NORMAL, "DDD", 0, 0);
+    textbox_set_ellipsize(row->textbox, lv->emode);
     box_add((box *)wid, WIDGET(row->textbox), TRUE);
   } else if (strcasecmp(label, "element-index") == 0) {
     row->index =
         textbox_create(WIDGET(wid), WIDGET_TYPE_TEXTBOX_TEXT, "element-index",
                        TB_AUTOHEIGHT, NORMAL, " ", 0, 0);
     box_add((box *)wid, WIDGET(row->index), FALSE);
+  } else if (strncasecmp(label, "textbox", 7) == 0) {
+    textbox *textbox_custom =
+        textbox_create(wid, WIDGET_TYPE_TEXTBOX_TEXT, label,
+                       TB_AUTOHEIGHT | TB_WRAP, NORMAL, "", 0, 0);
+    box_add((box *)wid, WIDGET(textbox_custom), TRUE);
+  } else if (strncasecmp(label, "button", 6) == 0) {
+    textbox *button_custom =
+        textbox_create(wid, WIDGET_TYPE_EDITBOX, label, TB_AUTOHEIGHT | TB_WRAP,
+                       NORMAL, "", 0, 0);
+    box_add((box *)wid, WIDGET(button_custom), TRUE);
+    widget_set_trigger_action_handler(WIDGET(button_custom),
+                                      textbox_button_trigger_action, lv->udata);
+  } else if (strncasecmp(label, "icon", 4) == 0) {
+    icon *icon_custom = icon_create(wid, label);
+    /* small hack to make it clickable */
+    const char *type =
+        rofi_theme_get_string(WIDGET(icon_custom), "action", NULL);
+    if (type) {
+      WIDGET(icon_custom)->type = WIDGET_TYPE_EDITBOX;
+    }
+    box_add((box *)wid, WIDGET(icon_custom), TRUE);
+    widget_set_trigger_action_handler(WIDGET(icon_custom),
+                                      textbox_button_trigger_action, lv->udata);
   } else {
     widget *wid2 = (widget *)box_create(wid, label, ROFI_ORIENTATION_VERTICAL);
     box_add((box *)wid, WIDGET(wid2), TRUE);
-    GList *list = rofi_theme_get_list(WIDGET(wid2), "children", "");
+    GList *list = rofi_theme_get_list_strings(
+        WIDGET(wid2),
+        "children"); // rofi_theme_get_list(WIDGET(wid2), "children", "");
     for (GList *iter = g_list_first(list); iter != NULL;
          iter = g_list_next(iter)) {
       listview_add_widget(lv, row, wid2, (const char *)iter->data);
@@ -188,11 +224,14 @@ static void listview_create_row(listview *lv, _listview_row *row) {
   row->box = box_create(WIDGET(lv), "element", ROFI_ORIENTATION_HORIZONTAL);
   widget_set_type(WIDGET(row->box), WIDGET_TYPE_LISTVIEW_ELEMENT);
   GList *list = NULL;
-  if (config.show_icons) {
-    list = rofi_theme_get_list(WIDGET(row->box), "children",
-                               "element-icon,element-text");
-  } else {
-    list = rofi_theme_get_list(WIDGET(row->box), "children", "element-text");
+  list = rofi_theme_get_list_strings(WIDGET(row->box), "children");
+  if (list == NULL) {
+    if (config.show_icons) {
+      list = g_list_append(list, g_strdup("element-icon"));
+      list = g_list_append(list, g_strdup("element-text"));
+    } else {
+      list = g_list_append(list, g_strdup("element-text"));
+    }
   }
 
   row->textbox = NULL;
@@ -206,7 +245,7 @@ static void listview_create_row(listview *lv, _listview_row *row) {
   g_list_free_full(list, g_free);
 }
 
-static int listview_get_desired_height(widget *wid);
+static int listview_get_desired_height(widget *wid, const int width);
 
 static void listview_free(widget *wid) {
   listview *lv = (listview *)wid;
@@ -247,6 +286,10 @@ static unsigned int scroll_per_page(listview *lv) {
         (lv->max_elements > 0) ? (lv->selected / lv->max_elements) : 0;
     offset = page * lv->max_elements;
     if (page != lv->cur_page) {
+
+      if (lv->page_callback)
+        lv->page_callback();
+
       lv->cur_page = page;
       lv->rchanged = TRUE;
     }
@@ -256,16 +299,19 @@ static unsigned int scroll_per_page(listview *lv) {
   return offset;
 }
 
-static unsigned int scroll_continious(listview *lv) {
-  unsigned int middle = (lv->max_rows - ((lv->max_rows & 1) == 0)) / 2;
+// For vertical packing flow
+static unsigned int scroll_continious_elements(listview *lv) {
+  unsigned int vmid = (lv->max_rows - 1) / 2;
+  unsigned int hmid = (lv->menu_columns - 1) / 2;
+  unsigned int middle = (lv->max_rows * hmid) + vmid;
   unsigned int offset = 0;
   if (lv->selected > middle) {
-    if (lv->selected < (lv->req_elements - (lv->max_rows - middle))) {
+    if (lv->selected < (lv->req_elements - (lv->max_elements - middle))) {
       offset = lv->selected - middle;
     }
     // Don't go below zero.
-    else if (lv->req_elements > lv->max_rows) {
-      offset = lv->req_elements - lv->max_rows;
+    else if (lv->req_elements > lv->max_elements) {
+      offset = lv->req_elements - lv->max_elements;
     }
   }
   if (offset != lv->cur_page) {
@@ -276,6 +322,31 @@ static unsigned int scroll_continious(listview *lv) {
   return offset;
 }
 
+// For horizontal packing flow
+static unsigned int scroll_continious_rows(listview *lv) {
+  unsigned int middle, selected, req_rows, offset;
+  middle = (lv->max_rows - 1) / 2;
+  selected = lv->selected / lv->menu_columns;
+  req_rows = (lv->req_elements + lv->menu_columns - 1) / lv->menu_columns;
+  offset = 0;
+  if (selected > middle) {
+    if (selected < (req_rows - (lv->max_rows - middle))) {
+      offset = selected - middle;
+    }
+    // Don't go below zero.
+    else if (req_rows > lv->max_rows) {
+      offset = req_rows - lv->max_rows;
+    }
+  }
+  offset *= lv->menu_columns;
+  if (offset != lv->cur_page) {
+    // scrollbar_set_handle ( lv->scrollbar, offset );
+    lv->cur_page = offset;
+    lv->rchanged = TRUE;
+  }
+  return offset;
+}
+
 static void update_element(listview *lv, unsigned int tb, unsigned int index,
                            gboolean full) {
   // Select drawing mode
@@ -319,7 +390,8 @@ static void barview_draw(widget *wid, cairo_t *draw) {
       if (lv->barview.direction == LEFT_TO_RIGHT) {
         for (unsigned int i = 0; i < max && width > 0; i++) {
           update_element(lv, i, i + offset, TRUE);
-          int twidth = widget_get_desired_width(WIDGET(lv->boxes[i].box));
+          int twidth = widget_get_desired_width(WIDGET(lv->boxes[i].box),
+                                                lv->element_height);
           if (twidth >= width) {
             if (!first) {
               break;
@@ -339,7 +411,8 @@ static void barview_draw(widget *wid, cairo_t *draw) {
         for (unsigned int i = 0;
              i < lv->cur_elements && width > 0 && i <= offset; i++) {
           update_element(lv, i, offset - i, TRUE);
-          int twidth = widget_get_desired_width(WIDGET(lv->boxes[i].box));
+          int twidth = widget_get_desired_width(WIDGET(lv->boxes[i].box),
+                                                lv->element_height);
           if (twidth >= width) {
             if (!first) {
               break;
@@ -378,10 +451,12 @@ static void barview_draw(widget *wid, cairo_t *draw) {
 static void listview_draw(widget *wid, cairo_t *draw) {
   unsigned int offset = 0;
   listview *lv = (listview *)wid;
-  if (lv->scroll_type == LISTVIEW_SCROLL_CONTINIOUS) {
-    offset = scroll_continious(lv);
-  } else {
+  if (lv->scroll_type == LISTVIEW_SCROLL_PER_PAGE) {
     offset = scroll_per_page(lv);
+  } else if (lv->pack_direction == ROFI_ORIENTATION_VERTICAL) {
+    offset = scroll_continious_elements(lv);
+  } else {
+    offset = scroll_continious_rows(lv);
   }
   // Set these all together to make sure they update consistently.
   scrollbar_set_max_value(lv->scrollbar, lv->req_elements);
@@ -426,30 +501,53 @@ static void listview_draw(widget *wid, cairo_t *draw) {
         }
       }
       for (unsigned int i = 0; i < max; i++) {
-        unsigned int ex =
-            left_offset + ((i) / lv->max_rows) * (element_width + spacing_hori);
+        if (lv->pack_direction == ROFI_ORIENTATION_HORIZONTAL) {
+          unsigned int ex = left_offset + ((i) % lv->cur_columns) *
+                                              (element_width + spacing_hori);
+          unsigned int ey = 0;
+          if (lv->reverse) {
+            ey = wid->h -
+                 (widget_padding_get_bottom(wid) +
+                  ((i) / lv->cur_columns) *
+                      (lv->element_height + spacing_vert)) -
+                 lv->element_height;
 
-        if ((i) / lv->max_rows == (lv->cur_columns - 1)) {
-          ex += d;
-        }
-        if (lv->reverse) {
-          unsigned int ey =
-              wid->h -
-              (widget_padding_get_bottom(wid) +
-               ((i) % lv->max_rows) * (lv->element_height + spacing_vert)) -
-              lv->element_height;
+            if ((i) / lv->cur_columns == (lv->cur_columns - 1)) {
+              ex += d;
+            }
+          } else {
+            ey = top_offset +
+                 ((i) / lv->cur_columns) * (lv->element_height + spacing_vert);
+
+            if ((i) / lv->cur_columns == (lv->cur_columns - 1)) {
+              ex += d;
+            }
+          }
           widget_move(WIDGET(lv->boxes[i].box), ex, ey);
           widget_resize(WIDGET(lv->boxes[i].box), element_width,
                         lv->element_height);
+
         } else {
-          unsigned int ey =
-              top_offset +
-              ((i) % lv->max_rows) * (lv->element_height + spacing_vert);
+          unsigned int ex = left_offset + ((i) / lv->max_rows) *
+                                              (element_width + spacing_hori);
+
+          if ((i) / lv->max_rows == (lv->cur_columns - 1)) {
+            ex += d;
+          }
+          unsigned int ey = 0;
+          if (lv->reverse) {
+            ey = wid->h -
+                 (widget_padding_get_bottom(wid) +
+                  ((i) % lv->max_rows) * (lv->element_height + spacing_vert)) -
+                 lv->element_height;
+          } else {
+            ey = top_offset +
+                 ((i) % lv->max_rows) * (lv->element_height + spacing_vert);
+          }
           widget_move(WIDGET(lv->boxes[i].box), ex, ey);
           widget_resize(WIDGET(lv->boxes[i].box), element_width,
                         lv->element_height);
         }
-
         update_element(lv, i, i + offset, TRUE);
         widget_draw(WIDGET(lv->boxes[i].box), draw);
       }
@@ -487,7 +585,14 @@ static void listview_recompute_elements(listview *lv) {
   }
   if (!(lv->fixed_columns) && lv->req_elements < lv->max_elements) {
     newne = lv->req_elements;
-    lv->cur_columns = (lv->req_elements + (lv->max_rows - 1)) / lv->max_rows;
+    if (lv->pack_direction == ROFI_ORIENTATION_VERTICAL) {
+      lv->cur_columns = (lv->req_elements + (lv->max_rows - 1)) / lv->max_rows;
+    } else {
+      lv->cur_columns = lv->menu_columns;
+      if (lv->req_elements < lv->menu_columns) {
+        lv->cur_columns = lv->req_elements;
+      }
+    }
   } else {
     newne = MIN(lv->req_elements, lv->max_elements);
     lv->cur_columns = lv->menu_columns;
@@ -519,6 +624,9 @@ void listview_set_num_elements(listview *lv, unsigned int rows) {
   }
   TICK_N("listview_set_num_elements");
   lv->req_elements = rows;
+  if (lv->require_input && !lv->filtered) {
+    lv->req_elements = 0;
+  }
   listview_set_selected(lv, lv->selected);
   TICK_N("Set selected");
   listview_recompute_elements(lv);
@@ -535,10 +643,20 @@ unsigned int listview_get_selected(listview *lv) {
 }
 
 void listview_set_selected(listview *lv, unsigned int selected) {
-  if (lv && lv->req_elements > 0) {
+  if (lv == NULL) {
+    return;
+  }
+  if (lv->req_elements > 0) {
     lv->selected = MIN(selected, lv->req_elements - 1);
     lv->barview.direction = LEFT_TO_RIGHT;
     widget_queue_redraw(WIDGET(lv));
+    if (lv->sc_callback) {
+      lv->sc_callback(lv, lv->selected, lv->sc_udata);
+    }
+  } else {
+    if (lv->sc_callback) {
+      lv->sc_callback(lv, UINT32_MAX, lv->sc_udata);
+    }
   }
 }
 
@@ -548,7 +666,12 @@ static void listview_resize(widget *wid, short w, short h) {
   lv->widget.h = MAX(0, h);
   int height = lv->widget.h - widget_padding_get_padding_height(WIDGET(lv));
   int spacing_vert = distance_get_pixel(lv->spacing, ROFI_ORIENTATION_VERTICAL);
-  lv->max_rows = (spacing_vert + height) / (lv->element_height + spacing_vert);
+  if (lv->widget.h == 0) {
+    lv->max_rows = lv->menu_lines;
+  } else {
+    lv->max_rows =
+        (spacing_vert + height) / (lv->element_height + spacing_vert);
+  }
   lv->max_elements = lv->max_rows * lv->menu_columns;
 
   widget_move(WIDGET(lv->scrollbar),
@@ -651,14 +774,15 @@ static gboolean listview_element_motion_notify(widget *wid,
   unsigned int i;
   for (i = 0; i < max && WIDGET(lv->boxes[i].box) != wid; i++) {
   }
-  if (i < max && i != listview_get_selected(lv)) {
+  if (i < max && (lv->last_offset + i) != listview_get_selected(lv)) {
     listview_set_selected(lv, lv->last_offset + i);
   }
   return TRUE;
 }
 
 listview *listview_create(widget *parent, const char *name,
-                          listview_update_callback cb, void *udata,
+                          listview_update_callback cb,
+                          listview_page_changed_cb page_cb, void *udata,
                           unsigned int eh, gboolean reverse) {
   listview *lv = g_malloc0(sizeof(listview));
   widget_init(WIDGET(lv), parent, WIDGET_TYPE_LISTVIEW, name);
@@ -685,26 +809,36 @@ listview *listview_create(widget *parent, const char *name,
       buff[i * 2] = 'a';
       buff[i * 2 + 1] = '\n';
     };
+    textbox_moveresize(row.textbox, 0, 0, 100000000, -1);
     textbox_text(row.textbox, buff);
   }
-  lv->element_height = widget_get_desired_height(WIDGET(row.box));
+  // Make textbox very wide.
+  lv->element_height = widget_get_desired_height(WIDGET(row.box), 100000);
   widget_free(WIDGET(row.box));
 
   lv->callback = cb;
   lv->udata = udata;
 
+  lv->page_callback = page_cb;
+
   // Some settings.
   lv->spacing = rofi_theme_get_distance(WIDGET(lv), "spacing", DEFAULT_SPACING);
   lv->menu_columns =
       rofi_theme_get_integer(WIDGET(lv), "columns", DEFAULT_MENU_COLUMNS);
+  lv->menu_lines =
+      rofi_theme_get_integer(WIDGET(lv), "lines", DEFAULT_MENU_LINES);
   lv->fixed_num_lines = rofi_theme_get_boolean(WIDGET(lv), "fixed-height",
                                                config.fixed_num_lines);
   lv->dynamic = rofi_theme_get_boolean(WIDGET(lv), "dynamic", TRUE);
   lv->reverse = rofi_theme_get_boolean(WIDGET(lv), "reverse", reverse);
+  lv->pack_direction =
+      rofi_theme_get_orientation(WIDGET(lv), "flow", ROFI_ORIENTATION_VERTICAL);
   lv->cycle = rofi_theme_get_boolean(WIDGET(lv), "cycle", config.cycle);
   lv->fixed_columns =
       rofi_theme_get_boolean(WIDGET(lv), "fixed-columns", FALSE);
 
+  lv->require_input =
+      rofi_theme_get_boolean(WIDGET(lv), "require-input", FALSE);
   lv->type = rofi_theme_get_orientation(WIDGET(lv), "layout",
                                         ROFI_ORIENTATION_VERTICAL);
   if (lv->type == LISTVIEW) {
@@ -732,6 +866,10 @@ static void listview_nav_up_int(listview *lv) {
   }
   lv->selected--;
   lv->barview.direction = RIGHT_TO_LEFT;
+
+  if (lv->sc_callback) {
+    lv->sc_callback(lv, lv->selected, lv->sc_udata);
+  }
   widget_queue_redraw(WIDGET(lv));
 }
 static void listview_nav_down_int(listview *lv) {
@@ -746,13 +884,55 @@ static void listview_nav_down_int(listview *lv) {
                      ? MIN(lv->req_elements - 1, lv->selected + 1)
                      : 0;
   lv->barview.direction = LEFT_TO_RIGHT;
+  if (lv->sc_callback) {
+    lv->sc_callback(lv, lv->selected, lv->sc_udata);
+  }
   widget_queue_redraw(WIDGET(lv));
 }
+void listview_nav_next(listview *lv) {
+  if (lv == NULL) {
+    return;
+  }
+  listview_nav_down_int(lv);
+}
+void listview_nav_prev(listview *lv) {
+  if (lv == NULL) {
+    return;
+  }
+  listview_nav_up_int(lv);
+}
+
+static void listview_nav_column_left_int(listview *lv) {
+  if (lv->selected >= lv->cur_columns) {
+    lv->selected -= lv->cur_columns;
+    if (lv->sc_callback) {
+      lv->sc_callback(lv, lv->selected, lv->sc_udata);
+    }
+    widget_queue_redraw(WIDGET(lv));
+  }
+}
+static void listview_nav_column_right_int(listview *lv) {
+  if ((lv->selected + lv->cur_columns) < lv->req_elements) {
+    lv->selected += lv->cur_columns;
+    if (lv->sc_callback) {
+      lv->sc_callback(lv, lv->selected, lv->sc_udata);
+    }
+    widget_queue_redraw(WIDGET(lv));
+  }
+}
 
 void listview_nav_up(listview *lv) {
   if (lv == NULL) {
     return;
   }
+  if (lv->pack_direction == ROFI_ORIENTATION_HORIZONTAL) {
+    if (lv->reverse) {
+      listview_nav_column_right_int(lv);
+    } else {
+      listview_nav_column_left_int(lv);
+    }
+    return;
+  }
   if (lv->reverse) {
     listview_nav_down_int(lv);
   } else {
@@ -763,6 +943,14 @@ void listview_nav_down(listview *lv) {
   if (lv == NULL) {
     return;
   }
+  if (lv->pack_direction == ROFI_ORIENTATION_HORIZONTAL) {
+    if (lv->reverse) {
+      listview_nav_column_left_int(lv);
+    } else {
+      listview_nav_column_right_int(lv);
+    }
+    return;
+  }
   if (lv->reverse) {
     listview_nav_up_int(lv);
   } else {
@@ -774,12 +962,22 @@ void listview_nav_left(listview *lv) {
   if (lv == NULL) {
     return;
   }
+  if (lv->max_rows == 0) {
+    return;
+  }
+  if (lv->pack_direction == ROFI_ORIENTATION_HORIZONTAL) {
+    listview_nav_up_int(lv);
+    return;
+  }
   if (lv->type == BARVIEW) {
     listview_nav_up_int(lv);
     return;
   }
   if (lv->selected >= lv->max_rows) {
     lv->selected -= lv->max_rows;
+    if (lv->sc_callback) {
+      lv->sc_callback(lv, lv->selected, lv->sc_udata);
+    }
     widget_queue_redraw(WIDGET(lv));
   }
 }
@@ -790,12 +988,19 @@ void listview_nav_right(listview *lv) {
   if (lv->max_rows == 0) {
     return;
   }
+  if (lv->pack_direction == ROFI_ORIENTATION_HORIZONTAL) {
+    listview_nav_down_int(lv);
+    return;
+  }
   if (lv->type == BARVIEW) {
     listview_nav_down_int(lv);
     return;
   }
   if ((lv->selected + lv->max_rows) < lv->req_elements) {
     lv->selected += lv->max_rows;
+    if (lv->sc_callback) {
+      lv->sc_callback(lv, lv->selected, lv->sc_udata);
+    }
     widget_queue_redraw(WIDGET(lv));
   } else if (lv->selected < (lv->req_elements - 1)) {
     // We do not want to move to last item, UNLESS the last column is only
@@ -807,6 +1012,9 @@ void listview_nav_right(listview *lv) {
     // If there is an extra column, move.
     if (col != ncol) {
       lv->selected = lv->req_elements - 1;
+      if (lv->sc_callback) {
+        lv->sc_callback(lv, lv->selected, lv->sc_udata);
+      }
       widget_queue_redraw(WIDGET(lv));
     }
   }
@@ -832,6 +1040,9 @@ static void listview_nav_page_prev_int(listview *lv) {
   } else {
     lv->selected -= (lv->max_elements);
   }
+  if (lv->sc_callback) {
+    lv->sc_callback(lv, lv->selected, lv->sc_udata);
+  }
   widget_queue_redraw(WIDGET(lv));
 }
 static void listview_nav_page_next_int(listview *lv) {
@@ -846,6 +1057,9 @@ static void listview_nav_page_next_int(listview *lv) {
     lv->selected = MIN(new, lv->req_elements - 1);
     lv->barview.direction = LEFT_TO_RIGHT;
 
+    if (lv->sc_callback) {
+      lv->sc_callback(lv, lv->selected, lv->sc_udata);
+    }
     widget_queue_redraw(WIDGET(lv));
     return;
   }
@@ -853,6 +1067,9 @@ static void listview_nav_page_next_int(listview *lv) {
   if (lv->selected >= lv->req_elements) {
     lv->selected = lv->req_elements - 1;
   }
+  if (lv->sc_callback) {
+    lv->sc_callback(lv, lv->selected, lv->sc_udata);
+  }
   widget_queue_redraw(WIDGET(lv));
 }
 
@@ -877,7 +1094,8 @@ void listview_nav_page_next(listview *lv) {
   }
 }
 
-static int listview_get_desired_height(widget *wid) {
+static int listview_get_desired_height(widget *wid,
+                                       G_GNUC_UNUSED const int width) {
   listview *lv = (listview *)wid;
   if (lv == NULL || lv->widget.enabled == FALSE) {
     return 0;
@@ -931,23 +1149,7 @@ void listview_set_mouse_activated_cb(listview *lv,
     lv->mouse_activated_data = udata;
   }
 }
-void listview_set_multi_select(listview *lv, gboolean enable) {
-  if (lv) {
-    lv->multi_select = enable;
-  }
-}
-void listview_set_num_lines(listview *lv, unsigned int num_lines) {
-  if (lv) {
-    lv->menu_lines = num_lines;
-  }
-}
 
-unsigned int listview_get_num_lines(listview *lv) {
-  if (lv) {
-    return lv->menu_lines;
-  }
-  return 0;
-}
 void listview_set_max_lines(listview *lv, unsigned int max_lines) {
   if (lv) {
     lv->max_displayed_lines = max_lines;
@@ -966,9 +1168,9 @@ void listview_set_fixed_num_lines(listview *lv) {
   }
 }
 
-void listview_set_ellipsize_start(listview *lv) {
+void listview_set_ellipsize(listview *lv, PangoEllipsizeMode mode) {
   if (lv) {
-    lv->emode = PANGO_ELLIPSIZE_START;
+    lv->emode = mode;
     for (unsigned int i = 0; i < lv->cur_elements; i++) {
       textbox_set_ellipsize(lv->boxes[i].textbox, lv->emode);
     }
@@ -991,3 +1193,15 @@ void listview_toggle_ellipsizing(listview *lv) {
     }
   }
 }
+
+void listview_set_filtered(listview *lv, gboolean filtered) {
+  if (lv) {
+    lv->filtered = filtered;
+  }
+}
+
+void listview_set_selection_changed_callback(
+    listview *lv, listview_selection_changed_callback cb, void *udata) {
+  lv->sc_callback = cb;
+  lv->sc_udata = udata;
+}
